from typing import Optional
import os

import click
from requests import Response

from zenodo_rest.entities import Deposition, Metadata
from zenodo_rest.entities.bucket_file import BucketFile

from . import actions


@click.group()
def depositions():
    pass


@depositions.command()
@click.option("--metadata", help="Optional json of metadata for the deposition.")
@click.option(
    "--metadata_file",
    type=click.Path(exists=True, file_okay=True, dir_okay=False),
    help="Optional json file of metadata for the deposition.",
)
@click.option(
    "--prereserve-doi",
    is_flag=True,
    help="Prereserve a DOI (not pushed to Datacite until deposition is published).",
)
@click.option("--dest", type=click.Path(), default=None, help="A file to write the resulting deposition json representation to.")
def create(
    metadata: Optional[str] = None,
    metadata_file: Optional[str] = None,
    prereserve_doi: Optional[bool] = None,
    dest: Optional[str] = None,
):

    metadata_parsed: Metadata = Metadata()
    if isinstance(metadata, str):
        metadata_parsed = Metadata.parse_raw(metadata)
    elif isinstance(metadata, Metadata):
        metadata_parsed = metadata

    if metadata_file is not None:
        metadata_parsed = Metadata.parse_file(metadata_file)

    deposition: Deposition = actions.create(metadata_parsed, prereserve_doi)
    json_response = deposition.json(exclude_none=True, indent=4)
    click.echo(json_response)
    if dest is None:
        return
    if len(os.path.dirname(dest)) > 0:
        os.makedirs(os.path.dirname(dest), exist_ok=True)
    with open(dest, 'w', encoding='utf-8') as f:
        f.write(json_response)


@depositions.command()
@click.argument("deposition-id", type=click.INT)
@click.option("--dest", type=click.Path(), default=None, help="A file to write the resulting deposition json representation to.")
def retrieve(deposition_id: int, dest: Optional[str] = None):
    """Retrieve deposition by ID from server.

    DEPOSITION-ID is the id of the deposition to be fetched
    """
    deposition: Deposition = actions.retrieve(deposition_id)
    json_response = deposition.json(exclude_none=True, indent=4)
    click.echo(json_response)
    if dest is None:
        return
    if len(os.path.dirname(dest)) > 0:
        os.makedirs(os.path.dirname(dest), exist_ok=True)
    with open(dest, 'w', encoding='utf-8') as f:
        f.write(json_response)


@depositions.command("list")
@click.option(
    "--query", "-q", help="Search query (using Elasticsearch query string syntax)."
)
@click.option(
    "--status", help="Filter result based on deposit status (either draft or published)"
)
@click.option(
    "--sort",
    help=(
        "Sort order (bestmatch or mostrecent)."
        "Prefix with minus to change form ascending to descending (e.g. -mostrecent)."
    ),
)
@click.option("--page", help="Page number for pagination")
@click.option("--size", help="Number of results to return per page.")
@click.option(
    "--all-versions",
    help="Show (true or 1) or hide (false or 0) all versions of deposits.",
)
def search_depositions(
    query: Optional[str] = None,
    status: Optional[str] = None,
    sort: Optional[str] = None,
    page: Optional[str] = None,
    size: Optional[int] = None,
    all_versions: bool = None,
):
    result: list[Deposition]
    result = actions.search(query, status, sort, page, size, all_versions)
    for x in result:
        click.echo(x.json(exclude_none=True, indent=2))


@depositions.command()
@click.argument("deposition-json",
                type=click.Path(exists=True, file_okay=True, dir_okay=False),
                )
@click.argument(
    "metadata_file",
    type=click.Path(exists=True, file_okay=True, dir_okay=False),
)
def update(
    deposition_json: str,
    metadata_file: str,
):
    """Update metadata for a not yet published deposition

    DEPOSITION_JSON the file with a json representation
    of the deposition to be updated

    METADATA_FILE the path to a metadata json file to be used as input
    """

    deposition: Deposition = Deposition.parse_file(deposition_json)
    deposition = deposition.get_latest()
    deposition = deposition.get_latest_draft()
    metadata = Metadata.parse_file(metadata_file)

    deposition = actions.update_metadata(deposition.id, metadata)
    json_response = deposition.json(exclude_none=True, indent=4)
    click.echo(json_response)


@depositions.command()
@click.argument("deposition-json",
                type=click.Path(exists=True, file_okay=True, dir_okay=False),
                )
def delete(
    deposition_json: str,
):
    """Delete a not yet published deposition

    DEPOSITION_JSON json representation of the deposition to be deleted
    """

    deposition: Deposition = Deposition.parse_file(deposition_json)
    deposition = deposition.get_latest()
    deposition = deposition.get_latest_draft()
    response: Response = actions.delete_remote(deposition.id)
    json_response = response.json(exclude_none=True, indent=4)
    click.echo(json_response)


@depositions.command()
@click.argument("deposition-json",
                type=click.Path(exists=True, file_okay=True, dir_okay=False),
                )
@click.argument(
    "file",
    type=click.Path(exists=True, file_okay=True, dir_okay=False),
)
def upload_file(
    deposition_json: str,
    file: str,
):
    """Upload a file to the bucket of a not yet published deposition

    DEPOSITION_JSON json representation of the deposition to be uploaded to.

    FILE the path to a file to be uploaded
    """
    deposition: Deposition = Deposition.parse_file(deposition_json)
    deposition = deposition.get_latest()
    deposition = deposition.get_latest_draft()
    bucket_file: BucketFile = actions.upload_file(deposition.id, file)
    json_response = bucket_file.json(exclude_none=True, indent=4)
    click.echo(json_response)


@depositions.command()
@click.argument("deposition-json",
                type=click.Path(exists=True, file_okay=True, dir_okay=False),
                )
@click.option("--dest", type=click.Path(), default=None, help="A file to write the resulting deposition json representation to.")
def publish(
        deposition_json: str,
        dest: Optional[str] = None,
):
    """Publish a pending deposition

    DEPOSITION_JSON json representation of the deposition to be published
    """

    deposition: Deposition = Deposition.parse_file(deposition_json)
    deposition = deposition.get_latest()
    deposition = deposition.get_latest_draft()
    deposition = actions.publish(deposition.id)
    json_response = deposition.json(exclude_none=True, indent=4)
    click.echo(json_response)
    if dest is None:
        return
    if len(os.path.dirname(dest)) > 0:
        os.makedirs(os.path.dirname(dest), exist_ok=True)
    with open(dest, 'w', encoding='utf-8') as f:
        f.write(json_response)


@depositions.command()
@click.argument("deposition-json",
                type=click.Path(exists=True, file_okay=True, dir_okay=False),
                )
@click.option("--dest", type=click.Path(), default=None, help="A file to write the resulting deposition json representation to.")
def new_version(
        deposition_json: str,
        dest: Optional[str] = None
):
    """Create a new version of a published disposition

    DEPOSITION_JSON json representation of the deposition to be published
    """

    deposition: Deposition = Deposition.parse_file(deposition_json)
    deposition = deposition.get_latest()
    deposition = actions.new_version(deposition.id)
    json_response = deposition.json(exclude_none=True, indent=4)
    click.echo(json_response)
    if dest is None:
        return
    if len(os.path.dirname(dest)) > 0:
        os.makedirs(os.path.dirname(dest), exist_ok=True)
    with open(dest, 'w', encoding='utf-8') as f:
        f.write(json_response)


@depositions.group()
def doi():
    """Get DOIs related to depositions"""
    pass


@doi.command()
@click.argument("deposition-json",
                type=click.Path(exists=True, file_okay=True, dir_okay=False),
                )
@click.option(
    "--full-url", "-f",
    is_flag=True,
    help="Return the full url of the latest draft's DOI",
)
def latest(
        deposition_json: str,
        full_url: bool,
):
    """Print the doi of the latest published version of the given deposition.

    DEPOSITION_JSON json representation of the deposition
    """

    deposition: Deposition = Deposition.parse_file(deposition_json)
    deposition = deposition.get_latest()
    if full_url:
        click.echo(deposition.doi_url)
    else:
        click.echo(deposition.doi)


@doi.command()
@click.argument("deposition-json",
                type=click.Path(exists=True, file_okay=True, dir_okay=False),
                )
@click.option(
    "--full-url", "-f",
    is_flag=True,
    help="Return the full url of the latest draft's DOI",
)
def latest_draft(
        deposition_json: str,
        full_url: bool,
):
    """Print the DOI of the latest related deposition draft

    DEPOSITION_JSON json representation of the deposition
    """

    deposition: Deposition = Deposition.parse_file(deposition_json)
    deposition = deposition.get_latest()
    deposition = deposition.get_latest_draft()
    if full_url:
        click.echo(deposition.doi_url)
    else:
        click.echo(deposition.doi)




from datetime import date
from enum import Enum
from typing import Optional, Union

from pydantic import BaseModel

from zenodo_rest.entities.community import Community
from zenodo_rest.entities.creator import Contributor, Creator
from zenodo_rest.entities.date_interval import DateInterval
from zenodo_rest.entities.doi import Doi
from zenodo_rest.entities.grant import Grant
from zenodo_rest.entities.location import Location
from zenodo_rest.entities.subject import Subject


class UploadType(str, Enum):
    publication = "publication"
    poster = "poster"
    presentation = "presentation"
    dataset = "dataset"
    image = "image"
    video_audio = "video"
    software = "software"
    lesson = "lesson"
    physical_object = "physicalobject"
    other = "other"


class PublicationType(str, Enum):
    annotation_collection = "annotationcollection"
    book = "book"
    section = "section"
    conference_paper = "conferencepaper"
    data_management_plan = "datamanagementplan"
    article = "article"
    patent = "patent"
    preprint = "preprint"
    deliverable = "deliverable"
    milestone = "milestone"
    proposal = "proposal"
    report = "report"
    software_documentation = "softwaredocumentation"
    taxonomic_treatment = "taxonomictreatment"
    technical_note = "technicalnote"
    thesis = "thesis"
    working_paper = "workingpaper"
    other = "other"


class ImageType(str, Enum):
    figure = "figure"
    plot = "plot"
    drawing = "drawing"
    diagram = "diagram"
    photo = "photo"
    other = "other"


class AccessRight(str, Enum):
    open = "open"
    embargoed = "embargoed"
    restricted = "restricted"
    closed = "closed"


class Metadata(BaseModel):
    upload_type: UploadType = UploadType.other
    publication_type: Optional[PublicationType] = None
    image_type: Optional[ImageType] = None
    publication_date: str = date.today().isoformat()
    title: str = "Placeholder"
    creators: list[Creator] = [Creator()]
    description: str = "Placeholder"
    access_right: AccessRight = AccessRight.open
    license: Optional[str] = None
    embargo_date: Optional[date] = None
    access_conditions: Optional[str] = None
    doi: Optional[str] = None
    prereserve_doi: Optional[Union[Doi, bool]] = None
    keywords: Optional[list[str]] = None
    notes: Optional[str] = None
    related_identifiers: Optional[list[object]] = None
    contributors: Optional[list[Contributor]] = None
    references: Optional[list[str]] = None
    communities: Optional[list[Community]] = None
    grants: Optional[list[Grant]] = None
    journal_title: Optional[str] = None
    journal_volume: Optional[str] = None
    journal_issue: Optional[str] = None
    journal_pages: Optional[str] = None
    conference_title: Optional[str] = None
    conference_acronym: Optional[str] = None
    conference_dates: Optional[str] = None
    conference_place: Optional[str] = None
    conference_url: Optional[str] = None
    conference_session: Optional[str] = None
    conference_session_part: Optional[str] = None
    imprint_publisher: Optional[str] = None
    imprint_isbn: Optional[str] = None
    imprint_place: Optional[str] = None
    partof_title: Optional[str] = None
    partof_pages: Optional[str] = None
    thesis_supervisors: Optional[list[Creator]] = None
    thesis_university: Optional[str] = None
    subjects: Optional[list[Subject]] = None
    version: Optional[str] = None
    language: Optional[str] = None
    locations: Optional[list[Location]] = None
    dates: Optional[list[DateInterval]] = None
    method: Optional[str] = None



from enum import Enum
from typing import Optional

from pydantic import BaseModel


class DateIntervalType(Enum):
    collected = "Collected"
    valid = "Valid"
    withdrawn = "Withdrawn"


class DateInterval(BaseModel):
    # at least start or end must be defined
    start: Optional[str] = None
    end: Optional[str] = None
    type: DateIntervalType
    description: Optional[str] = None



import os
from getpass import getpass

import click
from dotenv import load_dotenv

from zenodo_rest.depositions.depositions import depositions


@click.group()
@click.option(
    "--token",
    is_flag=True,
    show_default="ENVVAR: 'ZENODO_TOKEN'",
    help="Request a prompt to provide an authtoken to override ZENODO_TOKEN.",
)
@click.option(
    "--env",
    "-e",
    type=click.Path(exists=True, file_okay=True, dir_okay=False),
    help="Pass a path to a .env file to overwrite and add ENVVARS.",
)
def cli(token: bool = None, env: str = None):
    if env:
        load_dotenv(dotenv_path=env, override=True)
    if token:
        prompt = "Please enter your Zenodo token:"
        os.putenv("ZENODO_TOKEN", getpass(prompt))


cli.add_command(depositions)


def main():
    cli()


if __name__ == "__main__":
    main()



import os
from pathlib import Path
from typing import Optional

import requests

from zenodo_rest.entities.metadata import Metadata
from zenodo_rest.entities.deposition import Deposition
from zenodo_rest.entities.bucket_file import BucketFile


def create(
        metadata: Metadata = Metadata(),
        prereserve_doi: Optional[bool] = None,
        token: Optional[str] = None,
        base_url: Optional[str] = None,
) -> Deposition:
    """
    Create a deposition on the server, but do not publish it.
    """
    if token is None:
        token = os.getenv("ZENODO_TOKEN")
    if base_url is None:
        base_url = os.getenv("ZENODO_URL")

    if prereserve_doi is True:
        metadata.prereserve_doi = True

    header = {"Authorization": f"Bearer {token}"}
    response = requests.post(
        f"{base_url}/api/deposit/depositions",
        json={"metadata": metadata.dict(exclude_none=True)},
        headers=header,
    )

    response.raise_for_status()
    return Deposition.parse_obj(response.json())


def retrieve(
        deposition_id: str, token: Optional[str] = None, base_url: Optional[str] = None
) -> Deposition:
    return Deposition.retrieve(deposition_id, token, base_url)


def upload_file(
        deposition_id: str, path_to_file: str, token: Optional[str] = None
) -> BucketFile:
    deposition: Deposition = retrieve(deposition_id)
    bucket_url = deposition.get_bucket()
    if token is None:
        token = os.getenv("ZENODO_TOKEN")
    path = Path(path_to_file)
    header = {"Authorization": f"Bearer {token}"}
    with open(path_to_file, "rb") as fp:
        r = requests.put(
            f"{bucket_url}/{path.name}",
            data=fp,
            headers=header,
        )
    r.raise_for_status()
    return BucketFile.parse_obj(r.json())


def update_metadata(
        deposition_id: str,
        metadata: Metadata,
        token: Optional[str] = None,
        base_url: Optional[str] = None,
) -> Deposition:
    if token is None:
        token = os.getenv("ZENODO_TOKEN")
    if base_url is None:
        base_url = os.getenv("ZENODO_URL")
    header = {"Authorization": f"Bearer {token}", "Accept": "application/json"}

    response = requests.put(
        f"{base_url}/api/deposit/depositions/{deposition_id}",
        json={"metadata": metadata.dict(exclude_none=True)},
        headers=header,
    )

    response.raise_for_status()
    return Deposition.parse_obj(response.json())


def delete_remote(
        deposition_id: str, token: Optional[str] = None, base_url: Optional[str] = None
) -> requests.Response:
    if token is None:
        token = os.getenv("ZENODO_TOKEN")
    if base_url is None:
        base_url = os.getenv("ZENODO_URL")
    header = {
        "Authorization": f"Bearer {token}",
    }

    response = requests.delete(
        f"{base_url}/api/deposit/depositions/{deposition_id}",
        headers=header,
    )

    response.raise_for_status()
    return response


def publish(
        deposition_id: str, token: Optional[str] = None, base_url: Optional[str] = None
) -> Deposition:
    if token is None:
        token = os.getenv("ZENODO_TOKEN")
    if base_url is None:
        base_url = os.getenv("ZENODO_URL")
    header = {
        "Authorization": f"Bearer {token}",
    }

    response = requests.post(
        f"{base_url}/api/deposit/depositions/{deposition_id}/actions/publish",
        headers=header,
    )

    response.raise_for_status()
    return Deposition.parse_obj(response.json())


def new_version(
        deposition_id: str, token: Optional[str] = None, base_url: Optional[str] = None
) -> Deposition:
    if token is None:
        token = os.getenv("ZENODO_TOKEN", token)
    if base_url is None:
        base_url = os.getenv("ZENODO_URL")
    header = {
        "Authorization": f"Bearer {token}",
    }

    response = requests.post(
        f"{base_url}/api/deposit/depositions/{deposition_id}/actions/newversion",
        headers=header,
    )

    response.raise_for_status()
    deposition: Deposition = Deposition.parse_obj(response.json())
    return deposition


def search(
        query: Optional[str] = None,
        status: Optional[str] = None,
        sort: Optional[str] = None,
        page: Optional[str] = None,
        size: Optional[int] = None,
        all_versions: Optional[bool] = None,
        token: Optional[str] = None,
) -> list[Deposition]:

    if token is None:
        token = os.getenv("ZENODO_TOKEN")

    base_url = os.getenv("ZENODO_URL")
    header = {"Authorization": f"Bearer {token}"}
    params: dict = {}
    if query is not None:
        params["q"] = query
    if status is not None:
        params["status"] = status
    if sort is not None:
        params["sort"] = sort
    if page is not None:
        params["page"] = page
    if size is not None:
        params["size"] = size
    if all_versions:
        params["all_versions"] = "true"
    response = requests.get(
        f"{base_url}/api/deposit/depositions", headers=header, params=params
    )

    response.raise_for_status()
    return [Deposition.parse_obj(x) for x in response.json()]



from pydantic import BaseModel


class BucketFile(BaseModel):
    key: str # filename
    mimetype: str
    checksum: str
    version_id: str
    size: int
    created: str
    updated: str
    links: dict
    is_head: bool
    delete_marker: bool



from typing import Optional, TypeVar
import os

T = TypeVar('Deposition')

from pydantic import BaseModel
import requests

from zenodo_rest.entities.deposition_file import DepositionFile
from zenodo_rest.entities.metadata import Metadata


class Deposition(BaseModel):
    created: str
    doi: Optional[str]
    doi_url: Optional[str]
    files: Optional[list[DepositionFile]]
    id: str
    links: dict
    metadata: Metadata
    modified: str
    owner: int
    record_id: int
    record_url: Optional[str]
    state: str
    submitted: bool
    title: str

    @staticmethod
    def retrieve(
            deposition_id: str, token: Optional[str] = None, base_url: Optional[str] = None
    ) -> T:
        if token is None:
            token = os.getenv("ZENODO_TOKEN")
        if base_url is None:
            base_url = os.getenv("ZENODO_URL")
        header = {"Authorization": f"Bearer {token}", "Accept": "application/json"}

        response = requests.get(
            f"{base_url}/api/deposit/depositions/{deposition_id}",
            headers=header,
        )

        response.raise_for_status()
        return Deposition.parse_obj(response.json())

    def get_latest(self, token: str = None) -> Optional[T]:
        latest_url = self.links.get("latest", None)
        if latest_url is None:
            return self

        if token is None:
            token = os.getenv("ZENODO_TOKEN")

        header = {"Authorization": f"Bearer {token}", "Accept": "application/json"}
        response = requests.get(
            latest_url,
            headers=header,
        )

        response.raise_for_status()
        record = Deposition.parse_obj(response.json())
        return Deposition.retrieve(record.id)

    def get_latest_draft(self, token: str = None) -> Optional[T]:
        latest_draft_url = self.links.get("latest_draft", None)
        if latest_draft_url is None:
            return self

        if token is None:
            token = os.getenv("ZENODO_TOKEN")

        header = {"Authorization": f"Bearer {token}", "Accept": "application/json"}
        response = requests.get(
            latest_draft_url,
            headers=header,
        )
        response.raise_for_status()
        return Deposition.parse_obj(response.json())

    def get_bucket(self) -> str:
        return self.links.get('bucket')
